[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.4321715.svg)](https://doi.org/10.5281/zenodo.4321715)

# <img src="img/inPsightsLogo.png" alt="inPsights Logo" height="128"/>
**Chemical Insights from |Ψ|² Analysis**

inPsights is a open-source software package for the probability density analysis (PDA) of the total electronic wave function Ψ.
It analyzes maxima data generated by the open-source quantum Monte Carlo software package [Amolqc](https://github.com/luechow-group/Amolqc).

## Installation
### Installing required packages
To install inPsights, Git, CMake, the GNU Compiler Collection (gcc) must be installed. 
This is most easily achieved by employing a packet manager.

#### MacOS
Make sure that the Xcode Command Line Tools are installed already with: 
Make sure that the latest Xcode Command Line Tools are installed with: 
```bash
xcode-select --install
```

To install the required packages on MacOS, the [homebrew package manager](https://brew.sh) can be used. 
To download the packages with `homebrew` execute the following command in the terminal:
```bash
brew update
brew upgrade
brew install git cmake gcc@9 lapack eigen boost qt
```

Alternatively, the [Qt5 online installer](https://www.qt.io/download-open-source) can be used.
During the installation, make sure to install Qt for the `x86_64` architecture 
and select `sources`, `Qt3D`, and additionally `QtCharts` (which will be required in future versions as well).


#### Ubuntu
On Ubuntu, the package manager `aptitude` can be used.
To download the required packages with `aptitude` execute the following command in the terminal:
```bash
sudo apt -y install \
    build-essential git cmake \
    gcc g++ \
    libgomp1 libblas-dev liblapack-dev libeigen3-dev libboost-all-dev
```

In order to use the GUI, libgl-dev and Qt have to be installed:
```
sudo apt install libgl-dev
```
The preferred method to install Qt5 on Ubuntu is the Qt5 online installer, which is found on the [Qt webpage](https://www.qt.io/download-qt-installer).
Make sure to install the following components (Updated 13 August 2020. The component names tend to change.):
   * Desktop gcc 64-bit
   * Qt Charts
   * Qt Data Visualization

Alternatively, the package manager `aptitude` can be used:
```bash
sudo apt-get -y install qtbase5-dev qt3d5-dev 
```
This might cause problems during the build of the inPsights GUI.

### Setting environment variables

#### Compilers
Environment variables for different compilers and associated libraries can be specified e.g. 

for the `GNU Compiler Collection`
```bash
export CXX=/usr/local/bin/g++-10
```
or `Intel Parallel Studio XE`
```bash
export CXX=/opt/intel/bin/icpc
export INTELROOT=/opt/intel/
export MKLROOT=/opt/intel/mkl
```
Otherwise the default compilers are used.

#### Qt5
To build the inPsights GUI, environment variables to the Qt5 libraries are required.

If Qt5 is not found or a wrong Qt5 version is found, the following environment variable must be exported:

```bash
export CMAKE_PREFIX_PATH=/usr/local/Cellar/qt/5.XX.X
```

If Qt5 was installed via `apt-get`, CMake should find the library automatically.


## Building ProcessMaxima and inPsights

Create a build directory e.g.
```bash
mkdir cmake-build-release
cd cmake-build-release
```
and configure CMake for an out-of-source release build:
```bash
cmake ..
```
CMake options can be specified to build the GUI or to use a pre-compiled version of the Eigen library
```bash
cmake .. -DBUILD_GUI=ON -DBUILD_EIGEN=OFF
```

To build the `ProcessMaxima` executable run
```bash
make ProcessMaxima
```
To build the `inPsights` executable run
```bash
make inPsights
```
which requires `-DBUILD_GUI=ON`.

